<?php

/**
 * Helper class for theme functions.
 *
 * @since 1.0
 */
final class FLTheme {

	/**
	 * An array of data for each font to render.
	 *
	 * @since 1.0
	 * @access private
	 * @var array $fonts
	 */
	static private $fonts;

	/**
	 * Returns a Customizer setting.
	 *
	 * @since 1.0
	 *
	 * @param array|string $key The key of the setting to return.
	 * @param string $default The default if the settings is not set
	 *
	 * @return mixed
	 */
	static public function get_setting( $key = '', $default = '' )
	{
		$settings = FLCustomizer::get_mods();

		if ( isset( $settings[ $key ] ) ) {
			return $settings[ $key ];
		}
		else {
			return $default;
		}
	}

	/**
	 * Returns an array of all Customizer settings.
	 *
	 * @since 1.0
	 * @return array
	 */
	static public function get_settings()
	{
		return FLCustomizer::get_mods();
	}

	/**
	 * Checks to see if the current site is being accessed over SSL.
	 *
	 * @since 1.2.3
	 * @return bool
	 */
	static public function is_ssl()
	{
		if ( is_ssl() ) {
			return true;
		}
		else if ( 0 === stripos( get_option( 'siteurl' ), 'https://' ) ) {
			return true;
		}
		else if ( isset( $_SERVER['HTTP_X_FORWARDED_PROTO'] ) && 'https' == $_SERVER['HTTP_X_FORWARDED_PROTO'] ) {
			return true;
		}

		return false;
	}

	/**
	 * Theme setup logic called by the after_setup_theme action.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function setup()
	{
		global $content_width;

		// Localization (load as first thing before any translation texts)
		// Note: the first-loaded translation file overrides any following ones if the same translation is present.

		//wp-content/languages/theme-name/it_IT.mo
		load_theme_textdomain( 'wipi', trailingslashit( WP_LANG_DIR ) . 'themes/' . get_template() );

		//wp-content/themes/child-theme-name/languages/it_IT.mo
		load_theme_textdomain( 'wipi', get_stylesheet_directory() . '/languages' );

		//wp-content/themes/theme-name/languages/it_IT.mo
		load_theme_textdomain( 'wipi', get_template_directory() . '/languages' );

		// RSS feed links support
		add_theme_support('automatic-feed-links');

		// Title tag support
		add_theme_support('title-tag');

		// Post thumbnail support
		add_theme_support('post-thumbnails');

		// WooCommerce support
		add_theme_support('woocommerce');

		// Custom Logo
		add_theme_support( 'custom-logo', array(
			'height'      => 100,
			'width'       => 170,
			'flex-width'  => true,
			'flex-height' => true,
			'header-text' => array( 'site-title', 'site-description', 'fl-logo-text' ),
		) );

		// Nav menus
		register_nav_menus(array(
			'bar'     => __('Top Bar Menu', 'wipi'),
			'header'  => __('Header Menu', 'wipi'),
			'footer'  => __('Footer Menu', 'wipi')
		));

		// Include customizer settings.
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-general.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-header.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-content.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-footer.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-code.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-settings.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-presets.php';
		require_once WIPI_THEME_DIR . '/includes/customizer-panel-breadcrumb.php';


		$mods = FLCustomizer::get_mods();
		$content_width = isset($mods['fl-content-width']) ? $mods['fl-content-width'] : 0;

		add_action( 'admin_notices', 'FLTheme::plugin_notices' );

		// No CSS cache will make page load slower but allows for faster and convenient development.
		if ( ( defined( 'WIPI_NO_CACHE' ) && WIPI_NO_CACHE ) ) {
			FLCustomizer::refresh_css();
		}
	}

	static public function admin_actions() {
		if ( ! is_admin() || ! isset( $_GET['page'] ) || 'wipi-actions' !== $_GET['page'] ) {
			return;
		}

		if ( isset( $_GET['hide_notice'] ) ) {
			set_transient( 'wipi_hide_notice_' . $_GET['hide_notice'], true );
		}

		if ( wp_get_referer() ) {
			wp_safe_redirect( wp_get_referer() );
		} else {
			wp_safe_redirect( admin_url() );
		}
		exit;
	}


	static public function plugin_notices() {
		$notices = array();
		$errors  = array();
		$infos   = array();

		if ( defined( 'WIPI_NO_CACHE' ) && WIPI_NO_CACHE ) {
			$notices[ __( 'CSS cache is disabled', 'wipi' ) ] = sprintf(
				__( 'Your page load speed may be dramatically slower than it should. To re-enable CSS cache remove <code>%s</code> constant from your <em>wp-config.php</em> file.', 'wipi' ),
				'WIPI_NO_CACHE'
			);
		}

		if ( count( $notices ) ) {
			echo '<div class="notice notice-warning" style="background: #fcf8e3;">';
			foreach ( $notices as $title => $notice ) {
				echo '<p><strong>' . $title . '</strong> ' . $notice . '</p>';
			}
			echo '</div>';
		}

		if ( count( $infos ) ) {
			echo '<div class="notice notice-info" style="background: #DDF4FF;">';
			foreach ( $infos as $title => $notice ) {
				echo '<p><strong>' . $title . '</strong> ' . $notice . '</p>';
			}
			echo '</div>';
		}

		if ( count( $errors ) ) {
			echo '<div class="notice notice-error" style="background: #f2dede;">';
			foreach ( $errors as $title => $notice ) {
				echo '<p><strong>' . $title . '</strong> ' . $notice . '</p>';
			}
			echo '</div>';
		}
	}

	static public function upress_ad() {
		if ( get_user_meta( get_current_user_id(), 'upress_ad_dismissed', true ) ) {
			return true;
		}

		if ( DAY_IN_SECONDS > (time() - get_option( '_wipi_install_date', time() )) ) {
			return;
		}

		include WIPI_THEME_DIR . '/includes/ad-notice.php';
	}
	static public function upress_ad_scripts() {
		if ( get_user_meta( get_current_user_id(), 'upress_ad_dismissed', true ) ) {
			return true;
		}

		if ( DAY_IN_SECONDS > (time() - get_option( '_wipi_install_date', time() )) ) {
			return;
		}

		wp_enqueue_script( 'upress-ad', WIPI_THEME_URL . '/js/upress-ad.js', array( 'jquery' ), WIPI_THEME_VERSION, true );
		wp_localize_script( 'upress-ad', 'wipiobj', array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
			'nonce' => wp_create_nonce( 'upress-ad' ),
		) );
	}
	static public function upress_ad_dismiss() {
		check_ajax_referer( 'upress-ad' );

		update_user_meta( get_current_user_id(), 'upress_ad_dismissed', true );
	}

	static public function required_plugins() {
		tgmpa( array(
			array(
				'name'      => 'Wipi Builder',
				'slug'      => 'wipi-builder',
				'source'    => 'http://ws.wipi.co.il/api/download/latest/plugin/PJBlvq99WgtrFWENQHBJ9NjfQUvK7gdGH9aypGsNHivvrk0uUwS4RQlwEhGx/wipi-builder.zip',
				'is_callable' => 'FLBuilder::init',
				'required'  => false,
			),
			array(
				'name'      => 'TinyMCE Advanced',
				'slug'      => 'tinymce-advanced',
				'required'  => false,
			),
			array(
				'name'      => 'Enable Accessibility',
				'slug'      => 'enable-accessibility',
				'required'  => false,
			),
		), array(
			'id'           => 'wipi',
			'menu'         => 'tgmpa-install-plugins',
			'has_notices'  => true,
			'dismissable'  => true,
		) );
	}

	/**
	 * Enqueues theme styles and scripts.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function enqueue_scripts()
	{
		// Bootstrap
		wp_enqueue_style('bootstrap', WIPI_THEME_URL . '/css/bootstrap.min.css', array(), WIPI_THEME_VERSION);
		if( is_rtl() ) {
			wp_enqueue_style( 'bootstrap-rtl', WIPI_THEME_URL . '/css/bootstrap-rtl.min.css', array( 'bootstrap' ), WIPI_THEME_VERSION );
		}

		wp_enqueue_script( 'html5shiv', WIPI_THEME_URL . '/js/html5shiv.js', false, null );
		wp_script_add_data( 'html5shiv', 'conditional', 'lt IE 9' );
		wp_enqueue_script( 'respond', WIPI_THEME_URL . '/js/respond.min.js', false, null );
		wp_script_add_data( 'respond', 'conditional', 'lt IE 9' );

		// Fonts
		wp_enqueue_style('font-awesome', WIPI_THEME_URL . '/css/font-awesome.min.css', array(), WIPI_THEME_VERSION);
		wp_enqueue_style('mono-social-icons', WIPI_THEME_URL . '/css/mono-social-icons.css', array(), WIPI_THEME_VERSION);
		wp_enqueue_style( 'hebrew-fonts', WIPI_THEME_URL . '/css/hebrew-fonts.css', array(), WIPI_THEME_VERSION );
		self::fonts();

		// jQuery
		wp_enqueue_script('jquery');
		wp_enqueue_script('jquery-throttle', WIPI_THEME_URL . '/js/jquery.throttle.min.js', array('jquery'), WIPI_THEME_VERSION, true);

		// Lightbox
		if(self::get_setting('fl-lightbox') == 'enabled') {
			wp_enqueue_style('jquery-magnificpopup', WIPI_THEME_URL . '/css/jquery.magnificpopup.css', array(), WIPI_THEME_VERSION);
			wp_enqueue_script('jquery-magnificpopup', WIPI_THEME_URL . '/js/jquery.magnificpopup.min.js', array('jquery'), WIPI_THEME_VERSION, true);
		}

		// Threaded Comments
		if(is_singular() && comments_open() && get_option('thread_comments')) {
			wp_enqueue_script('comment-reply');
		}

		// Preview JS
		if(FLCustomizer::is_preset_preview()) {
			wp_enqueue_script('wipi-preview', WIPI_THEME_URL . '/js/preview.js', array(), WIPI_THEME_VERSION, true);
			wp_localize_script('wipi-preview', 'preview', array('preset' => $_GET['fl-preview']));
		}

		// Bootstrap and theme JS
		wp_enqueue_script('bootstrap', WIPI_THEME_URL . '/js/bootstrap.min.js', array('jquery'), WIPI_THEME_VERSION, true);
		wp_enqueue_script('wipi', WIPI_THEME_URL . '/js/theme.js', array('jquery', 'jquery-throttle'), WIPI_THEME_VERSION, true);

		// core stylesheet
		wp_enqueue_style( 'wipi', FLCustomizer::css_url(), array( 'bootstrap' ) );
		if( is_rtl() ) {
			wp_enqueue_style( 'wipi-rtl', WIPI_THEME_URL . '/css/rtl.css', array( 'wipi' ), WIPI_THEME_VERSION );
		}
	}

	static public function admin_enqueue_scripts() {
		if ( isset( $_GET['page'] ) && "wipi" === $_GET['page'] ) {
			wp_enqueue_style( 'bootstrap', WIPI_THEME_URL . '/css/bootstrap.min.css' );
			if ( is_rtl() ) {
				wp_enqueue_style( 'bootstrap', WIPI_THEME_URL . '/css/bootstrap-rtl.min.css' );
			}
			wp_enqueue_script( 'bootstrap', WIPI_THEME_URL . '/js/bootstrap.min.js', array( 'jquery' ), WIPI_THEME_VERSION, true );

			wp_enqueue_style( 'fl-theme-about', WIPI_THEME_URL . '/css/about.css' );
		}
	}

	static public function admin_pages() {
		global $submenu;

		if ( current_user_can( 'edit_theme_options' ) ) {
			if ( empty ( $GLOBALS['admin_page_hooks'][WIPI_THEME_MENU_SLUG] ) ) {
				add_menu_page(
					__( 'Wipi', 'wipi' ),
					__( 'Wipi', 'wipi' ),
					'edit_theme_options',
					WIPI_THEME_MENU_SLUG,
					'FLAdminAbout::render',
					'dashicons-groups'
				);
			}

			add_submenu_page(
				WIPI_THEME_MENU_SLUG,
				__( 'About Wipi', 'wipi' ),
				__( 'About Wipi', 'wipi' ),
				'edit_theme_options',
				WIPI_THEME_MENU_SLUG,
				'FLAdminAbout::render'
			);

			$permalink = admin_url( 'customize.php' ) . '?return=' . urlencode( $_SERVER['REQUEST_URI'] );
			$submenu[ WIPI_THEME_MENU_SLUG ][] = array( __( 'Customize', 'wipi' ), 'customize', $permalink );
		}
	}

	/**
	 * Initializes theme sidebars.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function widgets_init()
	{
		$footer_widgets_display = self::get_setting('fl-footer-widgets-display');
		$woo_layout             = self::get_setting('fl-woo-layout');
		$first_widget_install    = get_option( '_fl_widgets_initialized' );
		$first_wc_widget_install = get_option( '_fl_wc_widgets_initialized' );

		// Primary Sidebar
		register_sidebar(array(
			'name'          => __('Primary Sidebar', 'wipi'),
			'id'            => 'blog-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));
		register_sidebar(array(
			'name'          => __('Before Post Content', 'wipi'),
			'id'            => 'before-post-content-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));
		register_sidebar(array(
			'name'          => __('After Post Sidebar (Above comments)', 'wipi'),
			'id'            => 'after-post-above-comments-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));
		register_sidebar(array(
			'name'          => __('After Post Sidebar (Below comments)', 'wipi'),
			'id'            => 'after-post-below-comments-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));
		register_sidebar(array(
			'name'          => __('After Page Sidebar (Above comments)', 'wipi'),
			'id'            => 'after-page-above-comments-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));
		register_sidebar(array(
			'name'          => __('After Page Sidebar (Below comments)', 'wipi'),
			'id'            => 'after-page-below-comments-sidebar',
			'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="fl-widget-title">',
			'after_title'   => '</h4>'
		));

		// Footer Widgets
		if ( $footer_widgets_display != 'disabled' ) {
			register_sidebars( 4, array(
				'name'          => _x( 'Footer Column %d', 'Sidebar title. %d stands for the order number of the auto-created sidebar, 4 in total.', 'wipi' ),
				'id'            => 'footer-col',
				'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
				'after_widget'  => '</aside>',
				'before_title'  => '<h4 class="fl-widget-title">',
				'after_title'   => '</h4>'
			) );


			if ( ! $first_widget_install || ! $first_wc_widget_install ) {
				// add default widgets to footer on first install
				$active_widgets            = get_option( 'sidebars_widgets' );
				$widget_text_content       = array();
				$widget_categories_content = array();
				$widget_archives_content   = array();
				$widget_search_content     = array();
				$need_update               = false;

				if ( ! $first_widget_install ) {
					if ( empty ( $active_widgets['footer-col'] ) ) {
						$active_widgets['footer-col'][] = 'text-1';
						$widget_text_content[1]         = array(
							'title'  => __( 'Example Widget Title', 'wipi' ),
							'text'   => __( 'Example Widget Content', 'wipi' ),
							'filter' => ''
						);
						update_option( 'widget_text', $widget_text_content );
						$need_update = true;
					}
					if ( empty ( $active_widgets['footer-col-2'] ) ) {
						$active_widgets['footer-col-2'][] = 'categories-1';
						$widget_categories_content[1]     = array(
							'title'        => __( 'Categories', 'wipi' ),
							'count'        => 0,
							'hierarchical' => 0,
							'dropdown'     => 0
						);
						update_option( 'widget_categories', $widget_categories_content );
						$need_update = true;
					}
					if ( empty ( $active_widgets['footer-col-3'] ) ) {
						$active_widgets['footer-col-3'][] = 'archives-1';
						$widget_archives_content[1]       = array(
							'title'    => __( 'Archives', 'wipi' ),
							'count'    => 0,
							'dropdown' => 0
						);
						update_option( 'widget_archives', $widget_archives_content );
						$need_update = true;
					}
					if ( empty ( $active_widgets['footer-col-4'] ) ) {
						$active_widgets['footer-col-4'][] = 'search-1';
						$widget_search_content[1]         = array( 'title' => __( 'Search', 'wipi' ) );
						update_option( 'widget_search', $widget_search_content );
						$need_update = true;
					}

					update_option( '_fl_widgets_initialized', true );
				}


				if ( ! $first_wc_widget_install && class_exists( 'woocommerce' ) ) {
					// Woocommerce
					if ( ( ! isset( $active_widgets['woo-sidebar'] ) || ( isset( $active_widgets['woo-sidebar'] ) && count( $active_widgets['woo-sidebar'] ) <= 0 ) ) ) {
						$woo                                    = array();
						$woo['woocommerce_product_search']      = array(
							'title' => __( 'Product Search', 'wipi' )
						);
						$woo['woocommerce_layered_nav_filters'] = array(
							'title' => __( 'Active Filters', 'wipi' )
						);
						$woo['woocommerce_price_filter']        = array(
							'title' => __( 'Filter by price', 'wipi' )
						);
						$woo['woocommerce_recent_reviews']      = array(
							'title'  => __( 'Recent Reviews', 'wipi' ),
							'number' => 10
						);
						$woo['woocommerce_product_categories']  = array(
							'title'              => __( 'Product Categories', 'wipi' ),
							'orderby'            => 'name',
							'dropdown'           => 0,
							'count'              => 1,
							'hierarchical'       => 1,
							'show_children_only' => 0,
							'hide_empty'         => 0,
						);
						$woo['woocommerce_widget_cart']         = array(
							'title'         => __( 'Cart', 'wipi' ),
							'hide_if_empty' => 0
						);

						$i = 1;
						foreach ( $woo as $widget => $data ) {
							$active_widgets['woo-sidebar'][] = $widget . '-' . $i;
							update_option( 'widget_' . $widget, array( $i => $data ) );
							$i ++;
						}

						$need_update = true;
					}

					update_option( '_fl_wc_widgets_initialized', true );
				}


				if ( $need_update ) {
					update_option( 'sidebars_widgets', $active_widgets );
				}
			}

		}

		// WooCommerce Sidebar
		if ( $woo_layout != 'no-sidebar' && class_exists( 'Woocommerce' ) ) {
			register_sidebar( array(
				'name'          => __( 'WooCommerce Sidebar', 'wipi' ),
				'id'            => 'woo-sidebar',
				'before_widget' => '<aside id="%1$s" class="fl-widget %2$s">',
				'after_widget'  => '</aside>',
				'before_title'  => '<h4 class="fl-widget-title">',
				'after_title'   => '</h4>'
			) );
		}
	}

	static public function admin_head() {
		if ( isset( $_GET['page'] ) && "wipi" === $_GET['page'] ) {
			remove_all_actions( 'admin_notices' );
		}
	}

	/**
	 * Adds and renders the <link> tags for fonts.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function fonts()
	{
		$settings = self::get_settings();

		self::add_font( $settings['fl-body-font-family'], array( 300, 400, 700 ) );
		self::add_font( $settings['fl-heading-font-family'], $settings['fl-heading-font-weight'] );
		self::add_font( $settings['fl-nav-font-family'], $settings['fl-nav-font-weight'] );

		if ( $settings['fl-breadcrumb-show'] ) {
			self::add_font( $settings['fl-breadcrumb-title-font-family'], $settings['fl-breadcrumb-title-font-weight'] );
			self::add_font( $settings['fl-breadcrumb-nav-font-family'], $settings['fl-breadcrumb-nav-font-weight'] );
		}

		/*if ( $settings['fl-logo-type'] == 'text' ) {
			self::add_font( $settings['fl-logo-font-family'], $settings['fl-logo-font-weight'] );
		}*/

		self::render_fonts();
	}

	/**
	 * Adds data to the $fonts array for a font to be rendered.
	 *
	 * @since 1.0
	 * @param string $name The name key of the font to add.
	 * @param array $variants An array of weight variants.
	 * @return void
	 */
	static public function add_font($name, $variants = array())
	{
		$protocol   = self::is_ssl() ? 'https' : 'http';
		$google_url = $protocol . '://fonts.googleapis.com/css?family=';

		if(isset(self::$fonts[$name])) {
			foreach((array)$variants as $variant) {
				if(!in_array($variant, self::$fonts[$name]['variants'])) {
					self::$fonts[$name]['variants'][] = $variant;
				}
			}
		}
		else {
			self::$fonts[$name] = array(
				'url'      => isset(FLFontFamilies::$google[$name]) ? $google_url . $name : '',
				'variants' => (array)$variants
			);
		}
	}

	/**
	 * Renders the <link> tag for all fonts in the $fonts array.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function render_fonts()
	{
		foreach(self::$fonts as $name => $font) {
			if(!empty($font['url'])) {
				$subset = apply_filters( 'wipi_font_subset', '', $name );
				wp_enqueue_style( 'google-font-' . strtolower( $name ) , $font['url'] . ':'. implode(',', $font['variants']) . $subset, false, null );
			}
		}
	}

	/**
	 * Renders the <link> tags for theme CSS and any custom head code.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function head()
	{
		$settings = self::get_settings();

		// CSS
		if(!empty($settings['fl-css-code']) || FLCustomizer::is_customizer_preview()) {
			echo "<style id=\"fl-theme-custom-css\" type=\"text/css\">{$settings['fl-css-code']}</style>\n";
		}

		// JS
		if(!empty($settings['fl-js-code'])) {
			echo "<style id=\"fl-theme-custom-js\" type=\"text/css\">{$settings['fl-js-code']}</style>\n";
		}

		// Head
		if(!empty($settings['fl-head-code'])) {
			echo $settings['fl-head-code'] . "\n";
		}

		// Version info
		echo '<!-- ';
		echo 'Wipi theme v' . WIPI_THEME_VERSION;
		if ( defined( 'WIPI_BUILDER_VERSION' ) ) {
			echo ' with Wipi Builder v' . WIPI_BUILDER_VERSION;
		}
		echo " -->\n";

		do_action('wipi_head');
	}

	/**
	 * Adds custom CSS classes to the <body> tag.
	 * Called by the body_class filter.
	 *
	 * @since 1.0
	 * @param array $classes An array of the existing classes.
	 * @return array
	 */
	static public function body_class($classes)
	{
		$preset = self::get_setting('fl-preset');

		// Preset
		if ( empty( $preset ) ) {
			$classes[] = 'fl-preset-default';
		}
		else {
			$classes[] = 'fl-preset-' . $preset;
		}

		// Width
		if (
			( ( ! is_front_page() ) && ( self::get_setting( 'fl-layout-width' ) == 'full-width' ) )
			|| ( is_front_page() && ( self::get_setting( 'fl-front-page-layout-width' ) == 'full-width' ) )
		) {
			$classes[] = 'fl-full-width';
		}
		else {
			$classes[] = 'fl-fixed-width';
		}

		// Side navigation
		if ( 'no' === self::get_setting( 'fl-side-nav-enabled' ) ) {
			$classes[] = 'fl-disable-side-nav';
		}

		// Nav Vertical Left
		if(self::get_setting('fl-header-layout') == 'vertical-left') {
			$classes[] = 'fl-nav-vertical fl-nav-vertical-left';
		}

		// Nav Vertical Right
		if(self::get_setting('fl-header-layout') == 'vertical-right') {
			$classes[] = 'fl-nav-vertical fl-nav-vertical-right';
		}

		// Nav Transparent
		if(self::get_setting('fl-header-layout') == 'right-transparent') {
			$classes[] = 'fl-nav-right-transparent';
		}
		if(self::get_setting('transparent_only_frontpage') == 'enabled') {
			$classes[] = 'fl-nav-right-transparent-floating';
		}

		// Footer Parallax Effect
		if( (self::get_setting('fl-footer-parallax-effect') == 'enable') && (self::get_setting('fl-layout-width') == 'full-width') ) {
			$classes[] = 'fl-footer-effect';
		}

		// Nav Left
		if(self::get_setting('fl-header-layout') == 'left') {
			$classes[] = 'fl-nav-left';
		}

		// Shrink Fixed Header
		if( (self::get_setting('fl-fixed-header') == 'shrink') && (self::get_setting('fl-header-layout') != 'vertical-left') && (self::get_setting('fl-header-layout') != 'vertical-right') ) {
			$classes[] = 'fl-shrink';
		}

		// Fixed Header
		if( (self::get_setting('fl-fixed-header') == 'fixed') && (self::get_setting('fl-header-layout') != 'vertical-left') && (self::get_setting('fl-header-layout') != 'vertical-right') ) {
			$classes[] = 'fl-fixed-header';
		}

		// Hide Header Until Scroll
		if( (self::get_setting('fl-hide-until-scroll-header') == 'enable') && (self::get_setting('fl-fixed-header') == 'hidden') && (self::get_setting('fl-header-layout') != 'vertical-left') && (self::get_setting('fl-header-layout') != 'vertical-right') ) {
			$classes[] = 'fl-fixed-header';
			$classes[] = 'fl-scroll-header';
		}

		// fixed mobile header
		if( (self::get_setting('fl-mobile-nav-fixed-enabled') == '1') ) {
			$classes[] = 'fl-mobile-nav-fixed';
		}

		// Scroll To Top Button
		if(self::get_setting('fl-scroll-to-top') == 'enable') {
			$classes[] = 'fl-scroll-to-top';
		}


		if(self::get_setting('fl-header-nav-search') == 'visible') {
			$classes[] = 'fl-header-nav-search';
		}

		return apply_filters( 'wipi_body_classes', $classes );
	}

	/**
	 * Callback method for the nav menu fallback when no menu
	 * has been selected.
	 *
	 * @since 1.0
	 * @param array $args An array of args for the menu.
	 * @return void
	 */
	static public function nav_menu_fallback($args)
	{
		$url  = current_user_can('edit_theme_options') ? admin_url('nav-menus.php') : home_url();
		$text = current_user_can('edit_theme_options') ? __('Choose Menu', 'wipi') :  __('Home', 'wipi');

		echo '<ul class="fl-page-' . $args['theme_location'] . '-nav nav navbar-nav menu">';
		echo '<li>';
		echo '<a href="' . $url . '">' . $text . '</a>';
		echo '</li>';
		echo '</ul>';
	}

	/**
	 * Renders the markup for the top bar's first column.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function top_bar_col1()
	{
		$settings   = self::get_settings();
		$layout     = $settings['fl-topbar-layout'];
		$col_layout = $settings['fl-topbar-col1-layout'];
		$col_text   = $settings['fl-topbar-col1-text'];

		include locate_template( 'includes/top-bar-col1.php' );
	}

	/**
	 * Renders the markup for the top bar's second column.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function top_bar_col2()
	{
		$settings = self::get_settings();
		$layout     = $settings['fl-topbar-layout'];
		$col_layout = $settings['fl-topbar-col2-layout'];
		$col_text   = $settings['fl-topbar-col2-text'];

		include locate_template( 'includes/top-bar-col2.php' );
	}

	/**
	 * Renders the markup for the top bar.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function top_bar()
	{
		$top_bar_layout  = self::get_setting('fl-topbar-layout');
		$top_bar_enabled = apply_filters( 'wipi_topbar_enabled', true );

		if ( $top_bar_layout != 'none' && $top_bar_enabled ) {
			get_template_part( 'includes/top-bar' );
		}
	}

	/**
	 * Renders the custom code that is displayed before the page header.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function header_code()
	{
		echo self::get_setting('fl-header-code');
	}

	/**
	 * Renders the markup for the fixed header.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function fixed_header()
	{
		$header_layout  = self::get_setting( 'fl-fixed-header' );
		$nav_layout     = self::get_setting( 'fl-header-layout' );
		$header_enabled = apply_filters( 'wipi_fixed_header_enabled', true );
		if ( 'fadein' == $header_layout && $header_enabled && 'vertical-left' != $nav_layout && 'vertical-right' != $nav_layout) {
            get_template_part( 'includes/fixed-header' );
		}
	}

	/**
	 * Renders the markup for the main header.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function header_layout()
	{
		$header_layout  = self::get_setting( 'fl-header-layout' );
		$header_enabled = apply_filters( 'wipi_header_enabled', true );

		if ( 'none' != $header_layout && $header_enabled ) {
			get_template_part( 'includes/nav-' . $header_layout );
		}
	}

	/**
	 * Renders additional classes for the main header based on the
	 * selected header layout and mobile nav toggle.
	 *
	 * @since 1.3.1
	 * @return void
	 */
	static public function header_classes()
	{
		$header_layout   = self::get_setting( 'fl-header-layout' );
		$nav_toggle_type = self::get_setting( 'fl-mobile-nav-toggle' );

		echo ' fl-page-nav-' . $header_layout;
		echo ' fl-page-nav-toggle-' . $nav_toggle_type;
	}

	/**
	 * Renders additional data attributes for the main header
	 *
	 * @since 1.5
	 * @return void
	 */
	static public function header_data_attrs()
	{

		// Scroll Distance
		if(self::get_setting('fl-hide-until-scroll-header') == 'enable') {
			$scroll_distance = self::get_setting( 'fl-scroll-distance' );
			echo ' data-fl-distance=' . $scroll_distance;
		}

	}

	/**
	 * Renders the markup for the header content section.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function header_content()
	{
		$settings = self::get_settings();
		$layout   = $settings['fl-header-content-layout'];
		$text     = $settings['fl-header-content-text'];

		do_action( 'wipi_header_content_open' );

		if($layout == 'text' || $layout == 'social-text') {
			echo '<div class="fl-page-header-text">'. do_shortcode( $text ) .'</div>';
		}
		if($layout == 'social' || $layout == 'social-text') {
			self::social_icons();
		}

		do_action( 'wipi_header_content_close' );
	}

	/**
	 * Renders the header logo text or image.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function logo( $location = '' ) {
//        $fixed_header_custom_logo = self::get_setting( 'fl-custom-fixed-header-logo' );
//		$logo_type      = self::get_setting( 'fl-logo-type' );
//		$logo_image     = self::get_setting( 'fl-logo-image' );
//		$logo_retina    = self::get_setting( 'fl-logo-image-retina' );
//		$logo_text      = self::get_setting( 'fl-logo-text' );
//		$logo_width     = self::get_setting( 'fl-logo-width' );

		if( has_custom_logo() ) {
			the_custom_logo();
		} else {
			echo '<span class="fl-logo-text" itemprop="name">' . strip_tags( apply_filters( 'wipi_logo_text', get_bloginfo( 'name' ) ) ) . '</span>';
		}

//        if ( 'yes' === $fixed_header_custom_logo && 'fixed-header' === $location ) {
//			$logo_type = 'image';
//			$fixed_logo_image  = self::get_setting( 'fl-fixed-header-logo-image' );
//			$logo_retina = self::get_setting( 'fl-fixed-header-logo-image-retina' );
//			if($fixed_logo_image) {
//				$logo_image = $fixed_logo_image;
//			}
//		}
//
//		if ( empty( $logo_text ) || $logo_type == 'image' ) {
//			$logo_text = get_bloginfo( 'name' );
//		}
//
//		if ( empty( $logo_image ) ) {
//			$logo_type = 'text';
//		}
//
//		if ( $logo_type == 'image' ) {
//			echo '<img class="fl-logo-img" itemscope itemtype="http://schema.org/ImageObject" src="' . $logo_image . '"';
//			echo ' data-retina="' . $logo_retina . '"';
//                        echo $logo_width ? ' style="max-width: ' . $logo_width . '% !important"' : '';
//			echo ' alt="' . esc_attr( $logo_text ) . '" />';
//			echo '<meta itemprop="name" content="' . esc_attr( $logo_text ) . '" />';
//		} else {
//			echo '<span class="fl-logo-text" itemprop="name">' . do_shortcode( $logo_text ) . '</span>';
//		}
	}

	/**
	 * Renders the nav search icon and form.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function nav_search()
	{
		return;
		$nav_search = self::get_setting('fl-header-nav-search');

		if($nav_search == 'visible') {
			get_template_part('includes/nav-search');
		}
	}

	static public function search_menu_item( $items, $args ) {
		$nav_search = self::get_setting('fl-header-nav-search');
		if('header' != $args->theme_location || 'visible' != $nav_search) {
			return $items;
		}

		ob_start();
		get_template_part('includes/nav-search');
		$search = ob_get_clean();
		$items .= "<li id=\"menu-item-search\" class=\"menu-item menu-item-type-search menu-item-search\">{$args->before}{$search}{$args->after}</li>";

		return $items;
	}

	/**
	 * Renders the text for the mobile nav toggle button.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function nav_toggle_text()
	{
		$type = self::get_setting( 'fl-mobile-nav-toggle' );

		if ( 'icon' == $type ) {
			$text = '<i class="fa fa-bars"></i>';
		}
		else {
			$text = _x( 'Menu', 'Mobile navigation toggle button text.', 'wipi' );
		}

		echo apply_filters( 'wipi_nav_toggle_text', $text );
	}

	/**
	 * Renders a group of social icons whose URL has been set in the Customizer.
	 *
	 * @since 1.0
	 * @param bool $circle Whether to use circle icons or not.
	 * @return void
	 */
	static public function social_icons($circle = true)
	{
		$settings = self::get_settings();

		$keys = apply_filters( 'wipi_social_icons', array(
			'facebook',
			'twitter',
			'google',
			'linkedin',
			'yelp',
			'xing',
			'pinterest',
			'tumblr',
			'vimeo',
			'youtube',
			'flickr',
			'instagram',
			'dribbble',
			'500px',
			'blogger',
			'github',
			'rss',
			'email',
			'skype'
		) );

		$classs_map = apply_filters( 'wipi_social_icons_classes', array(
			'facebook' => 'fa-facebook',
			'twitter' => 'fa-twitter',
			'google' => 'fa-google-plus',
			'linkedin' => 'fa-linkedin',
			'yelp' => 'fa-yelp',
			'xing' => 'fa-xing',
			'pinterest' => 'fa-pinterest',
			'tumblr' => 'fa-tumblr',
			'vimeo' => 'fa-vimeo',
			'youtube' => 'fa-youtube',
			'flickr' => 'fa-flickr',
			'instagram' => 'fa-instagram',
			'dribble' => 'fa-dribble',
			'500px' => 'fa-500px',
			'blogger' => 'fa-blogger',
			'github' => 'fa-github',
			'rss' => 'fa-rss',
			'email' => 'fa-envelope',
			'skype' => 'fa-skype',
		) );

		echo '<div class="fl-social-icons">';

		foreach($keys as $key) {

			$link_target = ' target="_blank"';

			if(!empty($settings['fl-social-' . $key])) {

				$class = 'fl-social-icon fl-icon-color-'. $settings['fl-social-icons-color'] .' fl-icon-'. $key .' fl-social-icon-'. ($circle ? 'circle' : 'regular');

				if($key == 'email') {
					$settings['fl-social-' . $key] = 'mailto:' . $settings['fl-social-' . $key];
					$link_target = '';
				}

                if ( "skype" === $key ) {
					$settings[ 'fl-social-' . $key ] = "skype:" . $settings[ 'fl-social-' . $key ] . "?chat";
				}

				$icon_class = isset($classs_map[$key]) ? $classs_map[$key] : '';
				echo '<a href="'. $settings['fl-social-' . $key] . '"' . $link_target . ' class="'. $class .'" title="' . ucfirst( $key ) . '"><i class="fa fa-fw ' . $icon_class . '" role="presentation"></i></a>';
			}
		}

		echo '</div>';
	}

        /**
	 * Renders a group of social icons to share post.
	 *
	 * @since 1.0
	 *
	 * @param bool $circle Whether to use circle icons or not.
	 *
	 * @return void
	 */
	static public function post_sharing_icons( $circle = false ) {
		$url      = get_permalink();
		$settings = self::get_settings();
		$title    = get_the_title();

		$keys = apply_filters( 'wipi_social_icons', array(
			'facebook',
			'twitter',
			'google',
			'email',
			'pinterest',
			'linkedin',
			'reddit',
			'whatsapp',
		) );

		$classs_map = apply_filters( 'wipi_social_icons_classes', array(
			'facebook' => 'fa-facebook',
			'twitter' => 'fa-twitter',
			'google' => 'fa-google-plus',
			'email' => 'fa-envelope',
			'pinterest' => 'fa-pinterest-p',
			'linkedin' => 'fa-linkedin',
			'reddit' => 'fa-reddit-alien',
			'whatsapp' => 'fa-whatsapp',
		) );

		echo '<div class="fl-social-icons">';

		foreach ( $keys as $key ) {

			if('no' == FLTheme::get_setting("show-{$key}-share"))
				continue;

			$class = 'fa fa-fw ' . (isset($classs_map[$key]) ? $classs_map[$key] : 'fa-share-alt');

			switch($key) {
				case 'facebook':
					$link_target = 'https://facebook.com/sharer/sharer.php?u=' . $url;
					break;
				case 'twitter':
					$link_target = 'https://twitter.com/share?text=' . $title . '&url=' . $url;
					break;
				case 'google':
					$link_target = 'https://plus.google.com/share?url=' . $url;
					break;
				case 'email':
					$link_target = 'mailto:?body=' . $url;
					break;
				case 'pinterest':
					$link_target = 'https://pinterest.com/pin/create/button/?url=' . $url;
					break;
				case 'linkedin':
					$link_target = 'https://www.linkedin.com/shareArticle?mini=true&url=' . $url . '&title=' . $title . '&summary=' . $title;
					break;
				case 'reddit':
					$link_target = 'https://reddit.com/submit/?url=' . $url;
					break;
				case 'whatsapp':
					$link_target = 'whatsapp://send?text=' . $url;
					break;
				default:
					$link_target = "";
			}

			printf(
				'<a href="%s" class="fl-icon js-social-share-link" target="_blank" aria-label="%s"><i class="%s" role="presentation"></i></a>',
				$link_target,
				sprintf(
					__( 'Share on %s', 'wipi'),
					ucfirst($key)
				),
				$class
			);
		}

		echo '</div>';
	}


        /**
	 * Renders a div with author description / biography.
	 *
	 * @since 1.0
	 *
	 * @param bool $circle Whether to use circle icons or not.
	 *
	 * @return void
	 */
	static public function author_description() {
		$settings       = self::get_settings();
		$author_name    = get_the_author();
		$author_details = get_the_author_meta( 'description' );
		if ( empty( $author_details ) ) {
			$author_details = __( 'This box will show author description, text imported from WordPress User Profile -> User Description', 'wipi' );
		}
		$avatar = get_avatar( get_the_author_meta( 'ID' ), 60 );

		echo '<div class="fl-author-description">';
		echo '<div class="fl-author-title"><h4>' . $author_name . '</h4></div>';
		echo '<div class="fl-author-details">
					<div class="fl-author-avatar">' . $avatar . '</div>
					<div class="fl-author-details-text"><p>' . $author_details . '</p></div>
					</div>';
		echo '</div>';
	}


	/**
	 * Checks to see if the footer widgets and footer sections
	 * are enabled.
	 *
	 * @since 1.0
	 * @return bool
	 */
	static public function has_footer()
	{
		$footer_layout  = self::get_setting( 'fl-footer-layout' );
		$footer_enabled = apply_filters( 'wipi_footer_enabled', true );

		return $footer_enabled && ( self::has_footer_widgets() || $footer_layout != 'none' );
	}

	/**
	 * Renders the footer widgets section.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function footer_widgets()
	{
		if(self::has_footer_widgets()) {
			get_template_part('includes/footer-widgets');
		}
	}

	/**
	 * Checks to see if any footer widgets exist.
	 *
	 * @since 1.0
	 * @return bool
	 */
	static public function has_footer_widgets()
	{
		$show = self::get_setting('fl-footer-widgets-display');

		if($show == 'disabled' || (!is_front_page() && $show == 'home')) {
			return false;
		}

		for($i = 1; $i <= 4; $i++) {

			$id = $i == 1 ? 'footer-col' : 'footer-col-' . $i;

			if(is_active_sidebar($id)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Renders the columns and widgets for the footer widgets section.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function display_footer_widgets()
	{
		$active = array();
		$num_active = 0;

		for($i = 1; $i <= 4; $i++) {

			$id = $i == 1 ? 'footer-col' : 'footer-col-' . $i;

			if(is_active_sidebar($id)) {
				$active[] = $id;
				$num_active++;
			}
		}
		if($num_active > 0) {

			$col_length = 12/$num_active;

			for($i = 0; $i < $num_active; $i++) {
				echo '<div class="col-sm-' . $col_length . ' col-md-' . $col_length . '">';
				dynamic_sidebar($active[$i]);
				echo '</div>';
			}
		}
	}

	/**
	 * Renders the footer.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function footer()
	{
		$footer_layout = self::get_setting('fl-footer-layout');

		if($footer_layout != 'none') {
			get_template_part('includes/footer');
		}
	}

	/**
	 * Renders the footer's first column.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function footer_col1()
	{
		$settings   = self::get_settings();
		$layout     = $settings['fl-footer-layout'];
		$col_layout = $settings['fl-footer-col1-layout'];
		$col_text   = $settings['fl-footer-col1-text'];

		include locate_template( 'includes/footer-col1.php' );
	}

	/**
	 * Renders the footer's second column.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function footer_col2()
	{
		$settings   = self::get_settings();
		$layout     = $settings['fl-footer-layout'];
		$col_layout = $settings['fl-footer-col2-layout'];
		$col_text   = $settings['fl-footer-col2-text'];

		include locate_template( 'includes/footer-col2.php' );
	}

	/**
	 * Renders custom code that is displayed after the footer.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function footer_code()
	{
		echo self::get_setting('fl-footer-code');
	}

	/**
	 * Renders a theme sidebar.
	 *
	 * @since 1.0
	 * @param string $position The sidebar position, either left or right.
	 * @param string $section The section this sidebar belongs to.
	 * @return void
	 */
	static public function sidebar($position, $section = 'blog')
	{
		$size       = self::get_setting('fl-' . $section . '-sidebar-size');
		$display    = self::get_setting('fl-' . $section . '-sidebar-display');
		$layout     = self::get_setting('fl-' . $section . '-layout');

		if(strstr($layout, $position)) {
			include locate_template('sidebar.php');
		}
	}

	/**
	 * Renders the class for the main content wrapper.
	 *
	 * @since 1.0
	 * @param string $section The section this content belongs to.
	 * @return void
	 */
	static public function content_class($section = 'blog')
	{
		$layout       = self::get_setting('fl-' . $section . '-layout');
		$sidebar_size = self::get_setting('fl-' . $section . '-sidebar-size');
		$content_size = '8';

		if($sidebar_size == '2') {
			$content_size = '10';
		}
		elseif($sidebar_size == '3') {
			$content_size = '9';
		}

		if(strstr($layout, 'left')) {
			echo 'fl-content-right col-md-' . $content_size;
		}
		else if(strstr($layout, 'right')) {
			echo 'fl-content-left col-md-' . $content_size;
		}
		else {
			echo 'col-md-12';
		}
	}

	/**
	 * Renders the content header for post archives.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function archive_page_header()
	{
		// Category
		if ( is_category() ) {
			$page_title = single_cat_title( '', false );
		}
		// Tag
		else if ( is_tag() ) {
			$page_title = sprintf( _x( 'Posts Tagged &#8216;%s&#8217;', 'Archive title: tag.', 'wipi' ), single_tag_title( '', false ) );
		}
		// Day
		else if ( is_day() ) {
			$page_title = sprintf( _x( 'Archive for %s', 'Archive title: day.', 'wipi' ), get_the_date() );
		}
		// Month
		else if ( is_month() ) {
			$page_title = sprintf( _x( 'Archive for %s', 'Archive title: month.', 'wipi' ), single_month_title( ' ', false ) );
		}
		// Year
		else if ( is_year() ) {
			$page_title = sprintf( _x( 'Archive for %s', 'Archive title: year.', 'wipi' ), get_the_time( 'Y' ) );
		}
		// Author
		else if ( is_author() ) {
			$page_title = sprintf( _x( 'Posts by %s', 'Archive title: author.', 'wipi' ), get_the_author() );
		}
		// Search
		else if ( is_search() ) {
			$page_title = sprintf( _x( 'Search results for: %s', 'Search results title.', 'wipi' ), get_search_query() );
		}
		// Paged
		else if ( isset( $_GET['paged'] ) && ! empty( $_GET['paged'] ) ) {
			$page_title = _x( 'Archives', 'Archive title: paged archive.', 'wipi' );
		}
		// Index
		else {
			$page_title = '';
		}

		if(!empty($page_title)) {
			include locate_template('includes/archive-header.php');
		}
	}

	/**
	 * Renders the nav for post archives.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function archive_nav()
	{
		global $wp_query;

		if(function_exists('wp_pagenavi')) {
			wp_pagenavi();
		}
		elseif($wp_query->max_num_pages > 1) {
			echo '<nav class="fl-archive-nav clearfix">';
			echo '<div class="fl-archive-nav-prev">' . get_previous_posts_link(__('&laquo; Newer Posts', 'wipi')) . '</div>';
			echo '<div class="fl-archive-nav-next">' . get_next_posts_link(__('Older Posts &raquo;', 'wipi')) . '</div>';
			echo '</nav>';
		}
	}

	/**
	 * Renders the excerpt more text.
	 *
	 * @since 1.0
	 * @param string $more The existing more text.
	 * @return string
	 */
	static public function excerpt_more($more)
	{
		return '&hellip;';
	}

	/**
	 * Checks to see if the markup for the post content header
	 * should be rendered or not.
	 *
	 * @since 1.0
	 * @return bool
	 */
	static public function show_post_header()
	{
		if ( class_exists( 'FLBuilderModel' ) && FLBuilderModel::is_builder_enabled() ) {

			$global_settings = FLBuilderModel::get_global_settings();

			if ( ! $global_settings->show_default_heading ) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Renders the markup for the post top meta.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function post_top_meta()
	{
		global $post;

		$settings       = self::get_settings();
		$show_author    = $settings['fl-blog-post-author'] == 'visible' ? true : false;
		$show_date      = $settings['fl-blog-post-date'] == 'visible' ? true : false;
		$comments       = comments_open() || '0' != get_comments_number();
		$comment_count  = $settings['fl-blog-comment-count'] == 'visible' ? true : false;

		include locate_template('includes/post-top-meta.php');
	}

        /**
	 * Renders the markup for the post combined meta.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function post_combined_meta() {
		global $post;

		$settings    = self::get_settings();
		$show_author = $settings['fl-blog-post-author'] == 'visible' ? true : false;
		$show_date   = $settings['fl-blog-post-date'] == 'visible' ? true : false;
		$comments    = comments_open() || '0' != get_comments_number();
		$show_full   = $settings['fl-archive-show-full'];
		$show_cats   = $settings['fl-posts-show-cats'] == 'visible' ? true : false;
		$show_tags   = $settings['fl-posts-show-tags'] == 'visible' && get_the_tags() ? true : false;

		include locate_template( 'includes/post-combined-meta.php' );
	}


	/**
	 * Renders the markup for the post bottom meta.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function post_bottom_meta()
	{
		$settings  = self::get_settings();
		$show_full = $settings['fl-archive-show-full'];
		$show_cats = $settings['fl-posts-show-cats'] == 'visible' ? true : false;
		$show_tags = $settings['fl-posts-show-tags'] == 'visible' && get_the_tags() ? true : false;
		$comments  = comments_open() || '0' != get_comments_number();

		include locate_template('includes/post-bottom-meta.php');
	}

	/**
	 * Renders the markup for schema structured data.
	 *
	 * @since 1.4.1
	 * @return void
	 */
	static public function post_schema_meta()
	{
		// General Schema Meta
		echo '<meta itemscope itemprop="mainEntityOfPage" itemid="' . get_permalink() . '" />';
		echo '<meta itemprop="datePublished" content="' . get_the_time('Y-m-d') . '" />';
		echo '<meta itemprop="dateModified" content="' . get_the_modified_date('Y-m-d') . '" />';

		// Publisher Schema Meta
		echo '<div itemprop="publisher" itemscope itemtype="https://schema.org/Organization">';
		echo '<meta itemprop="name" content="' . esc_attr( get_bloginfo( 'name' ) ) . '">';

		if ( 'image' == self::get_setting( 'fl-logo-type' ) ) {
			echo '<div itemprop="logo" itemscope itemtype="https://schema.org/ImageObject">';

			$custom_logo_id = get_theme_mod( 'custom_logo' );
			$image = wp_get_attachment_image_src( $custom_logo_id , 'full' );

			echo '<meta itemprop="url" content="' . esc_attr( $image[0] ) . '">';
			echo '</div>';
		}

		echo '</div>';

		// Author Schema Meta
		echo '<div itemscope itemprop="author" itemtype="http://schema.org/Person">';
		echo '<meta itemprop="url" content="' . get_author_posts_url( get_the_author_meta( 'ID' ) ) . '" />';
		echo '<meta itemprop="name" content="' . get_the_author_meta( 'display_name', get_the_author_meta( 'ID' ) ) . '" />';
		echo '</div>';

		// Image Schema Meta
		if(has_post_thumbnail()) {

			$image = wp_get_attachment_image_src(get_post_thumbnail_id(get_the_ID()), 'full');

			if ( is_array( $image ) ) {
				echo '<div itemscope itemprop="image" itemtype="http://schema.org/ImageObject">';
				echo '<meta itemprop="url" content="' . $image[0] . '" />';
				echo '<meta itemprop="width" content="' . $image[1] . '" />';
				echo '<meta itemprop="height" content="' . $image[2] . '" />';
				echo '</div>';
			}
		}

		// Comment Schema Meta
		echo '<div itemprop="interactionStatistic" itemscope itemtype="http://schema.org/InteractionCounter">';
		echo '<meta itemprop="interactionType" content="http://schema.org/CommentAction" />';
		echo '<meta itemprop="userInteractionCount" content="' . wp_count_comments(get_the_ID())->approved . '" />';
		echo '</div>';
	}

	/**
	 * Renders the post nav for single post pages.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function post_navigation()
	{
		$show_nav = self::get_setting( 'fl-posts-show-nav' );

		if ( 'visible' == $show_nav ) {
			echo '<div class="fl-post-nav clearfix">';
			previous_post_link( '<span class="fl-post-nav-prev">%link</span>', '&larr; %title' );
			next_post_link( '<span class="fl-post-nav-next">%link</span>', '%title &rarr;' );
			echo '</div>';
		}
	}


        /**
	 * Renders the google analytics code.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function google_analytics_code() {
		echo self::get_setting( 'fl-analytics-code' );
	}

	/**
	 * Renders the Facebook Pixel code.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function facebook_pixel_code() {
		echo self::get_setting( 'fl-fbpixel-code' );
	}


	/**
	 * Renders the markup for each comment in a comments list.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function display_comment($comment, $args, $depth)
	{
		$GLOBALS['comment'] = $comment;

		include locate_template('includes/comment.php');
	}

	/**
	 * Initializes WooCommerce layout support.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function init_woocommerce()
	{
		remove_action('woocommerce_sidebar', 'woocommerce_get_sidebar', 10);
		remove_action('woocommerce_before_main_content', 'woocommerce_output_content_wrapper', 10);
		remove_action('woocommerce_after_main_content', 'woocommerce_output_content_wrapper_end', 10);

		add_action('woocommerce_before_main_content', 'FLTheme::woocommerce_wrapper_start', 10);
		add_action('woocommerce_after_main_content', 'FLTheme::woocommerce_wrapper_end', 10);
	}

	/**
	 * Renders the opening markup for WooCommerce pages.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function woocommerce_wrapper_start()
	{
		$layout = self::get_setting('fl-woo-layout');
		$col_size = $layout == 'no-sidebar' ? '12' : '8';

		echo '<div class="container">';
		echo '<div class="row">';
		self::sidebar('left', 'woo');
		echo '<div class="fl-content ';
		self::content_class('woo');
		echo '">';
	}

	/**
	 * Renders the closing markup for WooCommerce pages.
	 *
	 * @since 1.0
	 * @return void
	 */
	static public function woocommerce_wrapper_end()
	{
		$layout = self::get_setting('fl-woo-layout');

		echo '</div>';
		self::sidebar('right', 'woo');
		echo '</div>';
		echo '</div>';
	}

	/**
	 * Checks to see if a plugin is currently active.
	 *
	 * @since 1.0
	 * @param string $slug The slug of the plugin to check.
	 * @return bool
	 */
	static public function is_plugin_active( $slug )
	{
		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		return is_plugin_active( $slug . '/' . $slug . '.php' );
	}

	/**
	 * Renders scroll to top button for wp_footer.
	 *
	 * @since 1.5
	 * @return void
	 */
	static public function go_to_top()
	{
		if(self::get_setting('fl-scroll-to-top') == 'enable') {
			echo '<a href="#" id="fl-to-top"><i class="fa fa-fw fa-chevron-up"></i></a>';
		}
	}
}
